/**
 * Ce code fait partie des programmes d'exemple du cours de zttique
 * de l'Universit de Paris-Est Marne-la-Valle.
 *  
 *
 * Copyright (C) Universit Paris-Est Marne-la-Valle
 * @author Sebastien Paumier (paumier@univ-mlv.fr)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.umlv.zetetique.bible;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

@SuppressWarnings("serial")
public class CodeGrid extends JLayeredPane {

	JLabel[][] labels;
	int height,width;
	
	ArrayList<Line2D.Double> lines=new ArrayList<Line2D.Double>(); 
	ArrayList<Color> colors=new ArrayList<Color>();
	{
		colors.add(Color.CYAN);
		colors.add(Color.GREEN);
		colors.add(Color.YELLOW);
		colors.add(Color.PINK);
		colors.add(Color.ORANGE);
		colors.add(Color.MAGENTA);
	}
	
	BasicStroke stroke=new BasicStroke(20, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	Composite alpha=AlphaComposite.getInstance(AlphaComposite.SRC_ATOP,0.5f);
	Font highlightFont=new JLabel().getFont().deriveFont(Font.BOLD);
	
	JPanel panel=new JPanel(null);
	JPanel higherPane=new JPanel(null);
	
	JComponent glassPane=new JComponent() {
		@Override
		protected void paintComponent(Graphics g) {
			Graphics2D g2=(Graphics2D) g.create();
			try {
				g2.setStroke(stroke);
				g2.setComposite(alpha);
				int n=0;
				for (Line2D.Double line:lines) {
					g2.setColor(colors.get(n++));
					g2.draw(line);
				}
			} finally {g2.dispose();}
		}
	};
	
	public CodeGrid(int gridSize) {
		super();
		panel.setLayout(new GridLayout(gridSize,gridSize));
		this.height=gridSize;
		this.width=gridSize;
		labels=new JLabel[width][height];
		panel.setBackground(Color.WHITE);
		for (int i=0;i<width;i++) {
			for (int j=0;j<height;j++) {
				labels[i][j]=new JLabel();
				labels[i][j].setOpaque(true);
				labels[i][j].setBackground(Color.WHITE);
				labels[i][j].setPreferredSize(new Dimension(20,20));
				labels[i][j].setHorizontalAlignment(SwingConstants.CENTER);
				panel.add(labels[i][j]);
			}
		}
		add(panel,Integer.valueOf(0));
		add(glassPane,Integer.valueOf(1));
		higherPane.setOpaque(false);
		add(higherPane,Integer.valueOf(2));
		setLayout(new LayoutManager() {

			public void addLayoutComponent(String name, Component comp) {
				/* nop */
			}

			public void removeLayoutComponent(Component comp) {
				/* nop */
			}

			public Dimension preferredLayoutSize(Container parent) {
				Component[] components=parent.getComponents();
				if (components.length==0) {
					return new Dimension(0,0);
				}
				Dimension d=components[0].getPreferredSize();
				for (int i=1;i<components.length;i++) {
					Dimension tmp=components[i].getPreferredSize();
					if (d.width<tmp.width) d.width=tmp.width;
					if (d.height<tmp.height) d.height=tmp.height;
				}
				return d;
			}

			public Dimension minimumLayoutSize(Container parent) {
				Component[] components=parent.getComponents();
				if (components.length==0) {
					return new Dimension(0,0);
				}
				Dimension d=components[0].getMinimumSize();
				for (int i=1;i<components.length;i++) {
					Dimension tmp=components[i].getMinimumSize();
					if (d.width<tmp.width) d.width=tmp.width;
					if (d.height<tmp.height) d.height=tmp.height;
				}
				return d;
			}

			public void layoutContainer(Container parent) {
				Dimension d=parent.getSize();
				for (Component c:parent.getComponents()) {
					c.setSize(d);
				}
				if (SecretCode.prophecyPanel!=null) {
					SecretCode.prophecyPanel.refresh();
				}
			}
		});

	}
	
	public void clean() {
		display(null);
	}

	public void display(char[][] text) {
		lines.clear();
		glassPane.repaint();
		higherPane.removeAll();
		for (int row=0;row<height;row++) {
			for (int col=0;col<width;col++) {
				labels[row][col].setBackground(Color.WHITE);
				labels[row][col].setText(""+(text!=null?text[row][col]:" "));
			}
		}
	}
	
	static Random random=new Random();
	
	public void highlightWord(int index,String word,WordCoordinates wc) {
		Color c;
		if (index<colors.size()) {
			c=colors.get(index);
		} else {
			c=new Color(random.nextInt());
			colors.add(c);
		}
		int i;
		for (i=0;i<word.length();i++) {
			JLabel lbl=labels[wc.Y+i*wc.shiftY][wc.X+i*wc.shiftX];
			lbl.setBackground(Color.BLACK);
			JLabel lbl2=new JLabel(lbl.getText());
			lbl2.setSize(20,20);
			lbl2.setForeground(Color.WHITE);
			lbl2.setFont(highlightFont);
			lbl2.setHorizontalAlignment(SwingConstants.CENTER);
			higherPane.add(lbl2);
			lbl2.setBounds(lbl.getBounds());
		}
		i--;
		Rectangle r1=labels[wc.Y][wc.X].getBounds();
		Rectangle r2=labels[wc.Y+i*wc.shiftY][wc.X+i*wc.shiftX].getBounds();
		lines.add(new Line2D.Double(r1.getCenterX(),r1.getCenterY(),r2.getCenterX(),r2.getCenterY()));
	}
	
	public void refreshGlassPane() {
		glassPane.repaint();
	}
	
}
