/**
 * Ce code fait partie des programmes d'exemple du cours de zttique
 * de l'Universit de Paris-Est Marne-la-Valle.
 *  
 *
 * Copyright (C) Universit Paris-Est Marne-la-Valle
 * @author Sebastien Paumier (paumier@univ-mlv.fr)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.umlv.zetetique.bible;

import java.util.*;

public class Prophecy {

	public final static int MAX_PROPHECIES=10;
	public final static int MAX_COORD_PER_WORD=10;
	
	public char[][] grid;
	public int lineJump;
	public Occurrence mainWord;
	public WordCoordinates mainCoordinates;
	public ArrayList<GridWord> secondWords;
	
	public Prophecy(Occurrence w,WordCoordinates mainCoordinates,char[][] grid,int lineJump) {
		mainWord=w;
		this.mainCoordinates=mainCoordinates;
		this.grid=grid;
		this.lineJump=lineJump;
		secondWords=new ArrayList<GridWord>();
	}
	
	public GridWord lookForWord(String word) {
		GridWord gw=new GridWord(word);
		lookForSequence(word,gw);
		StringBuilder builder=new StringBuilder(word);
		lookForSequence(builder.reverse().toString(),gw);
		if (gw.positions.size()==0) return null;
		return gw;
	}

	public void lookForSequence(String word,GridWord gw) {
		int width=grid.length;
		int height=grid.length;
		int gridLength=width*height;
		for (int col=0;col<width;col++) {
			for (int row=0;row<height;row++) {
				if (grid[row][col]==word.charAt(0)) {
					/* We have a possible start point */
					int pos=row*width+col;
					for (int interval=1;interval<gridLength-1-(col*height+row);interval++) {
						boolean ok=true;
						boolean horizontalShift=((pos+interval)%width-col >= 0);
						int oldCol=col;
						int oldRow=row;
						int shiftX=-1,shiftY=-1;
						for (int z=1;z<word.length();z++) {
							int pos2=pos+z*interval;
							if (pos2>=gridLength) {
								ok=false;
								break;
							}
							int my_col=pos2%width;
							boolean ok2=((shiftX=(my_col-oldCol)) >= 0);
							if (ok2!=horizontalShift) {
								ok=false;
								break;
							}
							int my_row=pos2/width;
							shiftY=my_row-oldRow;
							if (grid[my_row][my_col]!=word.charAt(z)) {
								ok=false;
								break;
							}
							oldCol=my_col;
							oldRow=my_row;
						}
						if (ok) {
							if (gw.positions.size()==MAX_COORD_PER_WORD) {
								return;
							}
							gw.addCoordinates(col,row,shiftX,shiftY);
						}
						}
					}
				}
			}
		}
	
	
	public void addSecondWord(GridWord gw) {
		secondWords.add(gw);
	}
	
	
	@Override
	public String toString() {
		String s="***********************************\n"; 
		s=s+"word 0="+mainWord.word+" pos="+mainWord.start+" interval="+mainWord.interval+"\n";
		s=s+"   x="+mainCoordinates.X+" y="+mainCoordinates.Y+" shiftX="+mainCoordinates.shiftX+" shiftY="+mainCoordinates.shiftY+"\n";
		for (int i=0;i<secondWords.size();i++) {
			s=s+"-----------------\n";
			s=s+"word "+(i+1)+"="+secondWords.get(i).word+":\n";
			for (WordCoordinates wc:secondWords.get(i).positions) {
				s=s+"   x="+wc.X+" y="+wc.Y+" shiftX="+wc.shiftX+" shiftY="+wc.shiftY+"\n";
			}
		}
		s=s+"***********************************\n";
		return s;
	}
}

	

