/**
 * Ce code fait partie des programmes d'exemple du cours de zttique
 * de l'Universit de Paris-Est Marne-la-Valle.
 *  
 *
 * Copyright (C) Universit Paris-Est Marne-la-Valle
 * @author Sebastien Paumier (paumier@univ-mlv.fr)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */

package fr.umlv.zetetique.bible;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

public class SecretCode {

	public final static int WIDTH=40;
	
	public static ProphecyPanel prophecyPanel;
	public static JProgressBar progressBar=new JProgressBar();
	public static ArrayList<Prophecy> prophecies;
	public static JLabel nProphecies=new JLabel();
	
	public final static String[] names={"Penses de Pascal","Gnome de levure"};
	public final static String[] fileNames={"pascal.txt","genome.txt"};
	public final static String[] encodings={"US-ASCII","US-ASCII"};
	public final static boolean[] genomancy={false,true};
	
	
	public static int chooseText() {
		JPanel p = new JPanel();
		p.setLayout(new GridLayout(3, 1));
		p.setOpaque(true);
		JComboBox list = new JComboBox(names);
		p.add(new JLabel("Choose the text you want"));
		p.add(new JLabel("to work on:"));
		p.add(list);
		Object[] options = {"OK", "Exit"};
		if (1 == JOptionPane.showOptionDialog(null, p, "Secret Code Finder",
				JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null,
				options, options[0])) {
			System.exit(0);
		}
		System.out.println(list.getSelectedIndex());
		return list.getSelectedIndex();
	}
	
	static void refresh(final JFrame f, final CodeGrid grid, final int value) {
		if (prophecyPanel!=null) {
			f.getContentPane().remove(prophecyPanel);
		}
		f.getContentPane().add(prophecyPanel=new ProphecyPanel(prophecies.get(value),grid),BorderLayout.EAST);
		f.getContentPane().validate();
		prophecyPanel.revalidate();
		prophecyPanel.refresh();
		f.getContentPane().repaint();
	}
	
	/**
	 * @param args
	 * @throws IOException 
	 * @throws URISyntaxException 
	 */
	public static void main(String[] args) throws IOException, URISyntaxException {
		int n=chooseText();
		InputStream stream=SecretCode.class.getResourceAsStream("textes/"+fileNames[n]);
		final Text text=new Text(stream,encodings[n],genomancy[n]);
		final JFrame f=new JFrame("Secret Code Finder - http://zetetique.univ-mlv.fr");
		final CodeGrid grid=new CodeGrid(WIDTH);
		f.getContentPane().add(grid);
		final JSpinner spinner=new JSpinner(/*new SpinnerNumberModel(1,1,prophecies.size(),1)*/);
		spinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value=(Integer) spinner.getValue();
				refresh(f,grid,value-1);
			}

		});
		JPanel p=new JPanel();
		final JTextField textField=new JTextField(20);
		JButton search=new JButton("Search");
		progressBar.setPreferredSize(new Dimension(300,20));
		nProphecies.setPreferredSize(new Dimension(100,20));
		spinner.setEnabled(false);
		search.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				String s=textField.getText();
				if (s.equals("")) return;
				StringTokenizer tok=new StringTokenizer(s,", ;-.\t");
				if (tok.countTokens()==0) return;
				final String[] words=new String[tok.countTokens()];
				int i=0;
				while (tok.hasMoreTokens()) {
					String w=tok.nextToken().toUpperCase(Locale.ENGLISH);
					if (madeOfLetters(w)) {
						words[i++]=w;
					} else {
						return;
					}
				}
				grid.clean();
				nProphecies.setText("");
				new Thread(new Runnable() {
					public void run() {
						try {
							prophecies=text.findWords(WIDTH,words);
							if (prophecies==null || prophecies.size()==0) {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										nProphecies.setText("");
										progressBar.setString("No prophecy found");
										
								}});
								spinner.setEnabled(false);
								return;
							}
							SwingUtilities.invokeLater(new Runnable() {
								public void run() {
									nProphecies.setText(prophecies.size()+" prophec"+(prophecies.size()>1?"ies":"y"));
									spinner.setModel(new SpinnerNumberModel(1,1,prophecies.size(),1));
									spinner.setEnabled(true);
									refresh(f,grid,0);
								}
							});
						} catch (InterruptedException e) {
							e.printStackTrace();
						} catch (InvocationTargetException e1) {
							e1.printStackTrace();
						}
					}
				}).start();
			}
		});
		p.add(textField);
		p.add(search);
		p.add(progressBar);
		p.add(Box.createHorizontalGlue());
		p.add(nProphecies);
		p.add(spinner);
		f.getContentPane().add(p,BorderLayout.SOUTH);
		f.pack();
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		f.setVisible(true);
	}

	protected static boolean madeOfLetters(String w) {
		for (int i=0;i<w.length();i++) {
			char c=w.charAt(i);
			if (c<'A' || c>'Z') return false;
		}
		return true;
	}

}
