/**
 * Ce code fait partie des programmes d'exemple du cours de zttique
 * de l'Universit de Paris-Est Marne-la-Valle.
 *  
 *
 * Copyright (C) Universit Paris-Est Marne-la-Valle
 * @author Sebastien Paumier (paumier@univ-mlv.fr)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 */
package fr.umlv.zetetique.bible;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;

public class Text {

	private static final int MAX_LINE_JUMP=2;
	StringBuilder builder;
	Index index;
	
	
	public Text(InputStream stream,String encoding,boolean genome) throws IOException {
		InputStreamReader reader=new InputStreamReader(stream,encoding);
		int c;
		builder=new StringBuilder();
		if (!genome) {
			while ((c=reader.read())!=-1) {
				char t=(char)c;
				char s=filter(t);
				if (s!='\0') builder.append(s);
			}
		} else {
			readGenome(reader);
		}
		index=new Index(builder);
	}

	private void readGenome(InputStreamReader reader) throws IOException {
		char[] tab=new char[4];
		int current=0;
		int c;
		while ((c=reader.read())!=-1) {
			char t=(char)c;
			char s=filter(t);
			if (s=='A' || s=='C' || s=='G' || s=='T') {
				tab[current++]=s;
				if (current==4) {
					current=0;
					t=filter(getGenomeChar(tab));
					if (t!='\0') {
						builder.append(t);
					}
				}
			}
		}
	}

	private char getGenomeChar(char[] tab) {
		int n=0;
		for (int i=0;i<4;i++) {
			n=(n<<2)+getValue(tab[i]);
		}
		return (char)n;
	}

	private int getValue(char c) {
		switch(c) {
			case 'A': return 0;
			case 'C': return 1;
			case 'G': return 2;
			default: return 3;
		}
	}

	private char filter(char c) {
		if (c>='A' && c<='Z') return c;
		if (c>='a' && c<='z') return (char)(c-'a'+'A');
		switch (c) {
			case '\u00C0': case '\u00C2': case '\u00C4':  
			case '\u00E0': case '\u00E2': case '\u00E4': return 'A';
			
			case '\u00C9': case '\u00C8': case '\u00CA': case '\u00CB': 
			case '\u00E9': case '\u00E8': case '\u00EA': case '\u00EB': return 'E';

			case '\u00CE': case '\u00CF':  
			case '\u00EE': case '\u00EF': return 'I';
			
			case '\u00D4': case '\u00D6':  
			case '\u00F4': case '\u00F6': return 'O';
			
			case '\u00D9': case '\u00DB': case '\u00DC':  
			case '\u00F9': case '\u00FB': case '\u00FC': return 'U';
			
			case '\u00C7': case '\u00E7': return 'C';
		}
		return '\0';
	}
	

	public char getChar(int i) {
		if (i>=builder.length()) return ' ';
		return builder.charAt(i);
	}
	
	public int length() {
		return builder.length();
	}
	
	private String reverse(String s) {
		StringBuilder b=new StringBuilder(s);
		return b.reverse().toString();
	}
	
	
	public ArrayList<Prophecy> findWords(int WIDTH,final String... word) throws InterruptedException, InvocationTargetException {
		SwingUtilities.invokeAndWait(new Runnable() {
			public void run() {
				SecretCode.progressBar.setMaximum(0);
				SecretCode.progressBar.setValue(0);
				SecretCode.progressBar.setStringPainted(true);
				SecretCode.progressBar.setString("");
			}
		});
		final ArrayList<Prophecy> matches=new ArrayList<Prophecy>();
		final ArrayList<Occurrence> occurrences=findWord(word[0],word[0],0,WIDTH);
		occurrences.addAll(findWord(reverse(word[0]),word[0],occurrences.size(),WIDTH));
		SwingUtilities.invokeAndWait(new Runnable() {
			public void run() {
				SecretCode.progressBar.setMaximum(occurrences.size()-1);
				SecretCode.progressBar.setValue(0);
				SecretCode.progressBar.setStringPainted(true);
				SecretCode.progressBar.setString("Looking positions of "+word[0]);
			}
		});
		int n=0;
		for (Occurrence o:occurrences) {
			final int nn=n++;
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					SecretCode.progressBar.setValue(nn);
				}
			});
			for (int lineJump=0;lineJump<=MAX_LINE_JUMP;lineJump++) {
				//System.out.println("line jump="+lineJump);
				Prophecy prophecy;
				if ((prophecy=buildGrid(WIDTH,lineJump,o))==null) {
					/* if the word does not fit in the grid, we fail */
					break;
				}
				boolean ok=true;
				for (int i=1;i<word.length;i++) {
					GridWord gw=prophecy.lookForWord(word[i]);
					if (gw==null) {
						ok=false;
						break;
					}
					prophecy.addSecondWord(gw);
				}
				if (ok) {
					System.out.println(prophecy);
					SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
							SecretCode.nProphecies.setText(matches.size()+" prophec"+(matches.size()>1?"ies":"y"));
							
						}
					});
					matches.add(prophecy);
					if (matches.size()==Prophecy.MAX_PROPHECIES) {
						SwingUtilities.invokeAndWait(new Runnable() {
							public void run() {
								SecretCode.progressBar.setValue(SecretCode.progressBar.getMaximum());
								SecretCode.progressBar.setString("Done");
							}
						});
						return matches;
					}
				}
			}
		}
		SwingUtilities.invokeAndWait(new Runnable() {
			public void run() {
				SecretCode.progressBar.setValue(SecretCode.progressBar.getMaximum());
				SecretCode.progressBar.setString("Done");
			}
		});
		return matches;
	}
	
	
	private Prophecy buildGrid(int gridSize,int lineJump,Occurrence o) {
		int wordHeight=o.word.length()+lineJump*(o.word.length()-1);
		int heightShift=1+lineJump;
		int realInterval=o.interval/(1+lineJump);
		int widthShift=o.interval%(1+lineJump);
		int wordWidth=o.word.length()+(o.word.length()-1)*(widthShift-1);
		if (wordHeight>gridSize || wordWidth>gridSize) {
			/* If the word does not fit in the grid */
			System.err.println(o.word+" does not fit for line jump="+lineJump);
			return null;
		}
		char[][] grid=new char[gridSize][gridSize];
		int Y=(gridSize-wordHeight)/2;
		int X=(gridSize-wordWidth)/2;
		int firstCharPos=o.start-X-Y*realInterval;
		Prophecy prophecy=new Prophecy(o,new WordCoordinates(X,Y,widthShift,heightShift),grid,lineJump);
		for (int row=0;row<gridSize;row++) {
			for (int col=0;col<gridSize;col++) {
				int index1=firstCharPos+col;
				char c=(index1<0 || index1>=length())?' ':getChar(index1);
				grid[row][col]=c;
			}
			firstCharPos=firstCharPos+o.interval/(1+lineJump);
		}
		return prophecy;
	}
	
	

	public ArrayList<Occurrence> findWord(String word,final String wordToPrint,int size,int WIDTH) {
		ArrayList<Occurrence> list=new ArrayList<Occurrence>();
		int lessFreqLetterIndex=index.getLowestFreqLetterIndex(word);
		if (lessFreqLetterIndex==-1) {
			return list;
		}
		int[] foo=index.getIndices(word.charAt(lessFreqLetterIndex));
		int textLength=length();
		int wordLength=word.length();
		for (int start:foo) {
			for (int interval=WIDTH;interval<textLength-1;interval++) {
				int maxLength=(wordLength-1)*interval;
				int realStart=start-lessFreqLetterIndex*interval;
				if (realStart<0 || realStart+maxLength>textLength) break;
				boolean ok=true;
				for (int j=realStart,k=0;k<wordLength;j=j+interval,k++) {
					if (word.charAt(k)!=getChar(j)) {
						ok=false;
						break;
					}
				}
				if (ok) {
					list.add(new Occurrence(word,realStart,interval));
					final int n=size+list.size();
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							SecretCode.progressBar.setString(n+" "+wordToPrint+" found");
						}
					});
				}
			}
		}
		return list;
	}
}
